@@ -2,7 +2,7 @@
 
 #include "opencv2/opencv_modules.hpp"
 
-#if defined(HAVE_OPENCV_CUDACODEC) && defined(WIN32)
+#if defined(HAVE_OPENCV_CUDACODEC) && defined(_WIN32)
 
 #include <vector>
 #include <numeric>
